@@ -35,9 +35,15 @@ from bzrlib import (
     tests,
     )
 try:
-    from bzrlib.tests.features import Feature
+    from bzrlib.tests.features import (
+        Feature,
+        SymlinkFeature,
+        )
 except ImportError: # bzr < 2.5
-    from bzrlib.tests import Feature
+    from bzrlib.tests import (
+        Feature,
+        SymlinkFeature,
+        )
 
 from bzrlib.plugins.builddeb.import_dsc import (
         DistributionBranch,
@@ -904,7 +910,22 @@ class DistributionBranchTests(BuilddebTestCase):
         os.mkdir(basedir)
         write_to_file(os.path.join(basedir, "README"), "Hi\n")
         write_to_file(os.path.join(basedir, "BUGS"), "")
-        tar_path = "package_0.1.orig.tar.lzma"
+        # Some versions of tar, including that in Ubuntu lucid and maverick,
+        # but not natty and later, have a bug which prevents them from
+        # autodetecting the compression type of files less than 512 bytes in
+        # length. So, add some extra verbiage to push us just above that
+        # boundary. This matters for lzma, but not gz and bz2, because
+        # pristine-tar has its own decompression support for those.
+        write_to_file(os.path.join(basedir, "LOREM"), 
+                "Lorem ipsum dolor sit amet, consectetur adipisicing elit, "
+                "sed do eiusmod tempor incididunt ut labore et dolore magna "
+                "aliqua.  Ut enim ad minim veniam, quis nostrud exercitation "
+                "ullamco laboris nisi ut aliquip ex ea commodo consequat. "
+                "Duis aute irure dolor in reprehenderit in voluptate velit "
+                "esse cillum dolore eu fugiat nulla pariatur. Excepteur sint "
+                "occaecat cupidatat non proident, sunt in culpa qui officia "
+                "deserunt mollit anim id est laborum.")
+        tar_path = "package_0.1.orig.tar.xz"
         f = lzma.LZMAFile(tar_path, 'w')
         try:
             tf = tarfile.open(None, 'w', f)
@@ -926,7 +947,7 @@ class DistributionBranchTests(BuilddebTestCase):
         self.assertEqual(rev.message,
                 "Import upstream version %s" % str(version.upstream_version))
         self.assertEqual(rev.properties['deb-md5'], self.fake_md5_1)
-        self.assertTrue('deb-pristine-delta-lzma' in rev.properties)
+        self.assertTrue('deb-pristine-delta-xz' in rev.properties)
 
     def test_import_package_init_from_other(self):
         self.requireFeature(PristineTarFeature)
@@ -1818,7 +1839,7 @@ class DistributionBranchTests(BuilddebTestCase):
     def test_import_symlink(self):
         version = Version("1.0-1")
         self.requireFeature(PristineTarFeature)
-        self.requireFeature(tests.SymlinkFeature)
+        self.requireFeature(SymlinkFeature)
         builder = SourcePackageBuilder("package", version)
         builder.add_default_control()
         builder.add_upstream_symlink("a", "b")
