@@ -34,10 +34,12 @@ from bzrlib.revision import (
     Revision,
     )
 from bzrlib.tests import (
-    Feature,
     TestCase,
-    TestCaseWithTransport,
     )
+try:
+    from bzrlib.tests.features import Feature
+except ImportError: # bzr < 2.5
+    from bzrlib.tests import Feature
 from bzrlib.plugins.builddeb.config import (
     DebBuildConfig,
     )
@@ -46,15 +48,23 @@ from bzrlib.plugins.builddeb.errors import (
     PackageVersionNotPresent,
     WatchFileMissing,
     )
+from bzrlib.plugins.builddeb.tests import (
+    LzmaFeature,
+    TestCaseWithTransport,
+    )
 from bzrlib.plugins.builddeb.upstream import (
     AptSource,
-    PristineTarSource,
     StackedUpstreamSource,
     TarfileSource,
     UpstreamProvider,
     UpstreamSource,
     UScanSource,
-    Version,
+    extract_tarball_version,
+    gather_orig_files,
+    new_tarball_name,
+    )
+from bzrlib.plugins.builddeb.util import (
+    component_from_orig_tarball,
     )
 from bzrlib.plugins.builddeb.upstream.branch import (
     get_export_upstream_revision,
@@ -65,6 +75,11 @@ from bzrlib.plugins.builddeb.upstream.branch import (
     upstream_tag_to_version,
     upstream_version_add_revision
     )
+from bzrlib.plugins.builddeb.upstream.pristinetar import (
+    PristineTarSource,
+    is_upstream_tag,
+    upstream_tag_version,
+    )
 
 
 # Unless bug #712474 is fixed and available in the minimum bzrlib required, we
@@ -86,12 +101,14 @@ svn_plugin = SvnPluginAvailable()
 
 class MockSources(object):
 
-    def __init__(self, versions):
+    def __init__(self, versions, files):
         self.restart_called_times = 0
         self.lookup_called_times = 0
         self.lookup_package = None
         self.versions = versions
         self.version = None
+        self.filess = files
+        self.files = None
 
     def restart(self):
         self.restart_called_times += 1
@@ -102,9 +119,11 @@ class MockSources(object):
         self.lookup_package = package
         if self.lookup_called_times <= len(self.versions):
             self.version = self.versions[self.lookup_called_times-1]
+            self.files = self.filess[self.lookup_called_times-1]
             return True
         else:
             self.version = None
+            self.files = None
             return False
 
 
@@ -149,11 +168,11 @@ class AptSourceTests(TestCase):
 
     def test_apt_provider_no_package(self):
         caller = MockAptCaller()
-        sources = MockSources([])
+        sources = MockSources([], [])
         apt_pkg = MockAptPkg(sources)
         src = AptSource()
         src._run_apt_source = caller.call
-        self.assertRaises(PackageVersionNotPresent, src.fetch_tarball,
+        self.assertRaises(PackageVersionNotPresent, src.fetch_tarballs,
             "apackage", "0.2", "target", _apt_pkg=apt_pkg)
         self.assertEqual(1, apt_pkg.init_called_times)
         self.assertEqual(1, apt_pkg.get_pkg_source_records_called_times)
@@ -164,11 +183,12 @@ class AptSourceTests(TestCase):
 
     def test_apt_provider_wrong_version(self):
         caller = MockAptCaller()
-        sources = MockSources(["0.1-1"])
+        sources = MockSources(["0.1-1"],
+            [[("checksum", 0L, "apackage_0.1.orig.tar.gz", "tar")]])
         apt_pkg = MockAptPkg(sources)
         src = AptSource()
         src._run_apt_source = caller.call
-        self.assertRaises(PackageVersionNotPresent, src.fetch_tarball,
+        self.assertRaises(PackageVersionNotPresent, src.fetch_tarballs,
             "apackage", "0.2", "target", _apt_pkg=apt_pkg)
         self.assertEqual(1, apt_pkg.init_called_times)
         self.assertEqual(1, apt_pkg.get_pkg_source_records_called_times)
@@ -177,14 +197,56 @@ class AptSourceTests(TestCase):
         self.assertEqual("apackage", sources.lookup_package)
         self.assertEqual(0, caller.called)
 
-    def test_apt_provider_right_version(self):
+    def test_apt_provider_multiple_tarballs(self):
         caller = MockAptCaller(work=True)
-        sources = MockSources(["0.1-1", "0.2-1"])
+        sources = MockSources(["0.1-1", "0.2-1"],
+            [[("checksum", 0L, "apackage_0.1.orig.tar.gz", "tar")],
+             [("checksum", 0L, "apackage_0.2.orig.tar.bz2", "tar"),
+              ("checksum", 1L, "apackage_0.2.orig-extra.tar.gz", "tar")]])
         apt_pkg = MockAptPkg(sources)
         src = AptSource()
         src._run_apt_source = caller.call
-        src.fetch_tarball("apackage", "0.2", "target",
+        paths = src.fetch_tarballs("apackage", "0.2", "target",
             _apt_pkg=apt_pkg)
+        self.assertEquals(paths, [
+            "target/apackage_0.2.orig.tar.bz2",
+            "target/apackage_0.2.orig-extra.tar.gz"])
+
+    def test_apt_provider_right_version_bz2(self):
+        caller = MockAptCaller(work=True)
+        sources = MockSources(["0.1-1", "0.2-1"],
+            [[("checksum", 0L, "apackage_0.1.orig.tar.gz", "tar")],
+             [("checksum", 0L, "apackage_0.2.orig.tar.bz2", "tar")]])
+        apt_pkg = MockAptPkg(sources)
+        src = AptSource()
+        src._run_apt_source = caller.call
+        paths = src.fetch_tarballs("apackage", "0.2", "target",
+            _apt_pkg=apt_pkg)
+        self.assertEquals(paths, ["target/apackage_0.2.orig.tar.bz2"])
+
+    def test_apt_provider_right_version_xz(self):
+        caller = MockAptCaller(work=True)
+        sources = MockSources(["0.1-1", "0.2-1"],
+            [[("checksum", 0L, "apackage_0.1.orig.tar.gz", "tar")],
+             [("checksum", 0L, "apackage_0.2.orig.tar.xz", "tar")]])
+        apt_pkg = MockAptPkg(sources)
+        src = AptSource()
+        src._run_apt_source = caller.call
+        paths = src.fetch_tarballs("apackage", "0.2", "target",
+            _apt_pkg=apt_pkg)
+        self.assertEquals(paths, ["target/apackage_0.2.orig.tar.xz"])
+
+    def test_apt_provider_right_version(self):
+        caller = MockAptCaller(work=True)
+        sources = MockSources(["0.1-1", "0.2-1"],
+            [[("checksum", 0L, "apackage_0.1.orig.tar.gz", "tar")],
+             [("checksum", 0L, "apackage_0.2.orig.tar.gz", "tar")]])
+        apt_pkg = MockAptPkg(sources)
+        src = AptSource()
+        src._run_apt_source = caller.call
+        paths = src.fetch_tarballs("apackage", "0.2", "target",
+            _apt_pkg=apt_pkg)
+        self.assertEquals(paths, ["target/apackage_0.2.orig.tar.gz"])
         self.assertEqual(1, apt_pkg.init_called_times)
         self.assertEqual(1, apt_pkg.get_pkg_source_records_called_times)
         self.assertEqual(1, sources.restart_called_times)
@@ -198,24 +260,43 @@ class AptSourceTests(TestCase):
 
     def test_apt_provider_right_version_command_fails(self):
         caller = MockAptCaller()
-        sources = MockSources(["0.1-1", "0.2-1"])
+        sources = MockSources(["0.1-1", "0.2-1"],
+            [[("checksum", 0L, "apackage_0.1.orig.tar.gz", "tar")],
+             [("checksum", 0L, "apackage_0.2.orig.tar.gz", "tar")]])
         apt_pkg = MockAptPkg(sources)
         src = AptSource()
         src._run_apt_source = caller.call
-        self.assertRaises(PackageVersionNotPresent, src.fetch_tarball,
+        self.assertRaises(PackageVersionNotPresent, src.fetch_tarballs,
             "apackage", "0.2", "target",
             _apt_pkg=apt_pkg)
         self.assertEqual(1, apt_pkg.init_called_times)
         self.assertEqual(1, apt_pkg.get_pkg_source_records_called_times)
         self.assertEqual(1, sources.restart_called_times)
         # Only called twice means it stops when the command fails.
-        self.assertEqual(2, sources.lookup_called_times)
+        self.assertEqual(3, sources.lookup_called_times)
         self.assertEqual("apackage", sources.lookup_package)
         self.assertEqual(1, caller.called)
         self.assertEqual("apackage", caller.package)
         self.assertEqual("0.2-1", caller.version_str)
         self.assertEqual("target", caller.target_dir)
 
+    def test_apt_provider_right_version_is_native(self):
+        caller = MockAptCaller(work=True)
+        sources = MockSources(["0.1-1", "0.2-1"],
+            [[("checksum", 0L, "apackage_0.1.orig.tar.gz", "tar")],
+             [("checksum", 0L, "apackage_0.2-1.orig.tar.gz", "tar")]])
+        apt_pkg = MockAptPkg(sources)
+        src = AptSource()
+        src._run_apt_source = caller.call
+        self.assertRaises(PackageVersionNotPresent, src.fetch_tarballs,
+            "apackage", "0.2", "target", _apt_pkg=apt_pkg)
+        self.assertEqual(1, apt_pkg.init_called_times)
+        self.assertEqual(1, apt_pkg.get_pkg_source_records_called_times)
+        self.assertEqual(1, sources.restart_called_times)
+        self.assertEqual(3, sources.lookup_called_times)
+        self.assertEqual("apackage", sources.lookup_package)
+        self.assertEqual(0, caller.called)
+
 
 class RecordingSource(UpstreamSource):
 
@@ -227,11 +308,11 @@ class RecordingSource(UpstreamSource):
     def get_latest_version(self, package, current_version):
         return self._latest
 
-    def fetch_tarball(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir):
         self._specific_versions.append((package, version, target_dir))
         if not self._succeed:
             raise PackageVersionNotPresent(package, version, self)
-        return self._tarball_path(package, version, target_dir)
+        return [self._tarball_path(package, version, None, target_dir)]
 
     def __repr__(self):
         return "%s()" % self.__class__.__name__
@@ -239,12 +320,12 @@ class RecordingSource(UpstreamSource):
 
 class StackedUpstreamSourceTests(TestCase):
 
-    def test_fetch_tarball_first_wins(self):
+    def test_fetch_tarballs_first_wins(self):
         a = RecordingSource(False)
         b = RecordingSource(True)
         c = RecordingSource(False)
         stack = StackedUpstreamSource([a, b, c])
-        stack.fetch_tarball("mypkg", "1.0", "bla")
+        stack.fetch_tarballs("mypkg", "1.0", "bla")
         self.assertEquals([("mypkg", "1.0", "bla")], b._specific_versions)
         self.assertEquals([("mypkg", "1.0", "bla")], a._specific_versions)
         self.assertEquals([], c._specific_versions)
@@ -266,7 +347,7 @@ class StackedUpstreamSourceTests(TestCase):
         b = RecordingSource(False)
         stack = StackedUpstreamSource([a, b])
         self.assertRaises(PackageVersionNotPresent,
-                stack.fetch_tarball, "pkg", "1.0", "bla")
+                stack.fetch_tarballs, "pkg", "1.0", "bla")
         self.assertEquals([("pkg", "1.0", "bla")], b._specific_versions)
         self.assertEquals([("pkg", "1.0", "bla")], a._specific_versions)
 
@@ -281,7 +362,7 @@ class UScanSourceTests(TestCaseWithTransport):
         src = UScanSource(self.tree, False)
         self.assertRaises(WatchFileMissing, src._export_watchfile)
 
-    def test_export_watchfile_larstiq(self):
+    def test_export_watchfile_top_level(self):
         src = UScanSource(self.tree, True)
         self.build_tree(['watch'])
         self.assertRaises(WatchFileMissing, src._export_watchfile)
@@ -326,21 +407,21 @@ class UpstreamBranchSourceTests(TestCaseWithTransport):
         super(UpstreamBranchSourceTests, self).setUp()
         self.tree = self.make_branch_and_tree('.')
 
-    def test_fetch_tarball(self):
+    def test_fetch_tarballs(self):
         self.tree.commit("msg")
         self.tree.branch.tags.set_tag("1.0", self.tree.branch.last_revision())
         source = UpstreamBranchSource(self.tree.branch,
             {"1.0": self.tree.branch.last_revision()})
         os.mkdir("mydir")
-        self.assertEquals("mydir/foo_1.0.orig.tar.gz",
-            source.fetch_tarball("foo", "1.0", "mydir"))
-        self.failUnlessExists("mydir/foo_1.0.orig.tar.gz")
+        self.assertEquals(["mydir/foo_1.0.orig.tar.gz"],
+            source.fetch_tarballs("foo", "1.0", "mydir"))
+        self.assertPathExists("mydir/foo_1.0.orig.tar.gz")
 
-    def test_fetch_tarball_not_found(self):
+    def test_fetch_tarballs_not_found(self):
         source = UpstreamBranchSource(self.tree.branch)
         self.tree.commit("msg")
         self.assertRaises(PackageVersionNotPresent,
-            source.fetch_tarball, "foo", "1.0", "mydir")
+            source.fetch_tarballs, "foo", "1.0", "mydir")
 
     def test_get_latest_version(self):
         self.tree.commit("msg")
@@ -365,6 +446,24 @@ class UpstreamBranchSourceTests(TestCaseWithTransport):
             source.version_as_revision("foo", "2.1+bzr2"))
         self.assertEquals(revid1, source.version_as_revision("foo", "2.1"))
 
+    def test_version_as_revision_no_revspec(self):
+        # There is no relevant revspec known
+        config = DebBuildConfig(
+            [('user.conf', True), ('default.conf', False)],
+            branch=self.tree.branch)
+        source = UpstreamBranchSource(self.tree.branch, {}, config=config)
+        self.assertRaises(PackageVersionNotPresent,
+            source.version_as_revision, "foo", "2.1")
+
+    def test_version_as_revision_invalid_revspec(self):
+        # There is no relevant revspec known
+        config = DebBuildConfig(
+            [('user.conf', True), ('default.conf', False)],
+            branch=self.tree.branch)
+        source = UpstreamBranchSource(self.tree.branch, {}, config=config)
+        self.assertRaises(PackageVersionNotPresent,
+            source.version_as_revision, "foo", "2.1+bzr4242")
+
 
 class LazyUpstreamBranchSourceTests(TestCaseWithTransport):
     """Tests for LazyUpstreamBranchSource."""
@@ -373,24 +472,24 @@ class LazyUpstreamBranchSourceTests(TestCaseWithTransport):
         super(LazyUpstreamBranchSourceTests, self).setUp()
         self.tree = self.make_branch_and_tree('.')
 
-    def test_fetch_tarball(self):
+    def test_fetch_tarballs(self):
         self.tree.commit("msg")
         self.tree.branch.tags.set_tag("1.0", self.tree.branch.last_revision())
         source = LazyUpstreamBranchSource(self.tree.branch.base,
             {"1.0": self.tree.branch.last_revision()})
         self.assertIs(None, source._upstream_branch)
         os.mkdir("mydir")
-        self.assertEquals("mydir/foo_1.0.orig.tar.gz",
-            source.fetch_tarball("foo", "1.0", "mydir"))
-        self.failUnlessExists("mydir/foo_1.0.orig.tar.gz")
+        self.assertEquals(["mydir/foo_1.0.orig.tar.gz"],
+            source.fetch_tarballs("foo", "1.0", "mydir"))
+        self.assertPathExists("mydir/foo_1.0.orig.tar.gz")
         self.assertIsNot(None, source._upstream_branch)
 
-    def test_fetch_tarball_not_found(self):
+    def test_fetch_tarballs_not_found(self):
         source = LazyUpstreamBranchSource(self.tree.branch.base)
         self.assertIs(None, source._upstream_branch)
         self.tree.commit("msg")
         self.assertRaises(PackageVersionNotPresent,
-            source.fetch_tarball, "foo", "1.0", "mydir")
+            source.fetch_tarballs, "foo", "1.0", "mydir")
         self.assertIsNot(None, source._upstream_branch)
 
     def test_get_latest_version(self):
@@ -582,6 +681,34 @@ class GetRevisionSnapshotTests(TestCase):
         self.assertEquals("svn:2424", get_snapshot_revision("0.4.4+svn2424"))
 
 
+class TestIsUpstreamTag(TestCase):
+
+    def test_plain_version(self):
+        self.assertFalse(is_upstream_tag('2.1'))
+
+    def test_simple_upstream(self):
+        self.assertTrue(is_upstream_tag('upstream-2.1'))
+
+    def test_distro_upstream(self):
+        self.assertTrue(is_upstream_tag('upstream-debian-2.1'))
+
+    def test_git_upstream(self):
+        self.assertTrue(is_upstream_tag('upstream/2.1'))
+
+
+class TestUpstreamTagVersion(TestCase):
+
+    def test_simple_upstream(self):
+        self.assertEqual('2.1', upstream_tag_version('upstream-2.1'))
+
+    def test_distro_upstream(self):
+        self.assertEqual('2.1',
+            upstream_tag_version('upstream-debian-2.1'))
+
+    def test_git_upstream(self):
+        self.assertEqual('2.1', upstream_tag_version('upstream/2.1'))
+
+
 class PristineTarSourceTests(TestCaseWithTransport):
 
     def setUp(self):
@@ -596,13 +723,18 @@ class PristineTarSourceTests(TestCaseWithTransport):
                 "upstream-" + upstream_v_no)
 
     def test_tag_name_distro(self):
-        self.assertEquals(self.source.tag_name("0.3", "ubuntu"),
+        self.assertEquals(self.source.tag_name("0.3", distro="ubuntu"),
                 "upstream-ubuntu-0.3")
 
     def test_version(self):
         self.assertEquals(['upstream-3.3', 'upstream-debian-3.3',
             'upstream-ubuntu-3.3', 'upstream/3.3'],
-            self.source.possible_tag_names("3.3"))
+            self.source.possible_tag_names("3.3", component=None))
+
+    def test_version_component(self):
+        self.assertEquals(['upstream-3.3/extlib', 'upstream-debian-3.3/extlib',
+            'upstream-ubuntu-3.3/extlib'],
+            self.source.possible_tag_names("3.3", component="extlib"))
 
     def test_pristine_tar_format_gz(self):
         rev = Revision("myrevid")
@@ -614,6 +746,11 @@ class PristineTarSourceTests(TestCaseWithTransport):
         rev.properties["deb-pristine-delta-bz2"] = "1"
         self.assertEquals("bz2", self.source.pristine_tar_format(rev))
 
+    def test_pristine_tar_format_xz(self):
+        rev = Revision("myrevid")
+        rev.properties["deb-pristine-delta-xz"] = "1"
+        self.assertEquals("xz", self.source.pristine_tar_format(rev))
+
     def test_pristine_tar_format_unknown(self):
         rev = Revision("myrevid")
         self.assertRaises(AssertionError,
@@ -651,40 +788,69 @@ class TarfileSourceTests(TestCaseWithTransport):
         source = TarfileSource("foo-1.0.tar.gz", "1.0")
         self.assertEquals("1.0", source.get_latest_version("foo", "0.9"))
 
-    def test_fetch_tarball(self):
+    def test_get_latest_version_parses(self):
+        source = TarfileSource("foo-1.0.tar.gz")
+        self.assertEquals("1.0", source.get_latest_version("foo", "0.9"))
+
+    def test_fetch_tarballs(self):
         source = TarfileSource("foo-1.0.tar.gz", "1.0")
         os.mkdir("bar")
-        self.assertEquals("bar/foo_1.0.orig.tar.gz",
-            source.fetch_tarball("foo", "1.0", "bar"))
-        self.failUnlessExists("bar/foo_1.0.orig.tar.gz")
+        self.assertEquals(["bar/foo_1.0.orig.tar.gz"],
+            source.fetch_tarballs("foo", "1.0", "bar"))
+        self.assertPathExists("bar/foo_1.0.orig.tar.gz")
 
-    def test_fetch_tarball_repack(self):
+    def test_fetch_zip_tarballs_repack(self):
         zf = zipfile.ZipFile("bla-2.0.zip", "w")
         zf.writestr('avoid', 'empty zip to make the repacker happy\n')
         zf.close()
         source = TarfileSource("bla-2.0.zip", "2.0")
         os.mkdir("bar")
-        self.assertEquals("bar/foo_2.0.orig.tar.gz",
-            source.fetch_tarball("foo", "2.0", "bar"))
-        self.failUnlessExists("bar/foo_2.0.orig.tar.gz")
+        self.assertEquals(["bar/foo_2.0.orig.tar.gz"],
+            source.fetch_tarballs("foo", "2.0", "bar"))
+        self.assertPathExists("bar/foo_2.0.orig.tar.gz")
 
-    def test_fetch_tarball_not_present(self):
+    def test_fetch_tarballs_not_present(self):
         source = TarfileSource("foo-1.0.tar.gz", "1.0")
         os.mkdir("bar")
         self.assertRaises(PackageVersionNotPresent,
-            source.fetch_tarball, "foo", "0.9", "bar")
+            source.fetch_tarballs, "foo", "0.9", "bar")
 
-    def test_fetch_tarball_bz2(self):
+    def test_fetch_tarballs_bz2(self):
         tar = tarfile.open("foo-1.0.tar.bz2", "w:bz2")
         tar.close()
         # verify this is a bzip2 file
-        bz2.BZ2File("foo-1.0.tar.bz2").close()
+        os.mkdir("foo-1.0")
+        zf = bz2.BZ2File("foo-1.0.tar.bz2", 'w')
+        try:
+            tar = tarfile.open("foo-1.0.tar", "w", zf)
+            try:
+                tar.add("foo-1.0")
+            finally:
+                tar.close()
+        finally:
+            zf.close()
         source = TarfileSource("foo-1.0.tar.bz2", "1.0")
         os.mkdir("bar")
-        self.assertEquals("bar/foo_1.0.orig.tar.gz",
-            source.fetch_tarball("foo", "1.0", "bar"))
-        self.failUnlessExists("bar/foo_1.0.orig.tar.gz")
-        gzip.open("bar/foo_1.0.orig.tar.gz").close()
+        self.assertEquals(["bar/foo_1.0.orig.tar.bz2"],
+            source.fetch_tarballs("foo", "1.0", "bar"))
+        self.assertPathExists("bar/foo_1.0.orig.tar.bz2")
+        tarfile.open("bar/foo_1.0.orig.tar.bz2", "r:bz2").close()
+
+    def test_fetch_tarball_xz(self):
+        self.requireFeature(LzmaFeature)
+        import lzma
+        lzma_f = lzma.LZMAFile("foo-1.0.tar.xz", 'w')
+        try:
+            tar = tarfile.open("foo-1.0.tar", "w", lzma_f)
+            tar.close()
+        finally:
+            lzma_f.close()
+        source = TarfileSource("foo-1.0.tar.xz", "1.0")
+        os.mkdir("bar")
+        self.assertEquals(["bar/foo_1.0.orig.tar.xz"],
+            source.fetch_tarballs("foo", "1.0", "bar"))
+        self.assertPathExists("bar/foo_1.0.orig.tar.xz")
+        lzma.LZMAFile("bar/foo_1.0.orig.tar.xz").close()
 
 
 class _MissingUpstreamProvider(UpstreamProvider):
@@ -694,7 +860,7 @@ class _MissingUpstreamProvider(UpstreamProvider):
         pass
 
     def provide(self, target_dir):
-        raise MissingUpstreamTarball("test_tarball")
+        raise MissingUpstreamTarball("test_tarball", "1.0")
 
 
 class _TouchUpstreamProvider(UpstreamProvider):
@@ -704,9 +870,11 @@ class _TouchUpstreamProvider(UpstreamProvider):
         self.desired_tarball_name = desired_tarball_name
 
     def provide(self, target_dir):
-        f = open(os.path.join(target_dir, self.desired_tarball_name), "wb")
+        path = os.path.join(target_dir, self.desired_tarball_name)
+        f = open(path, "wb")
         f.write("I am a tarball, honest\n")
         f.close()
+        return [path]
 
 
 class _SimpleUpstreamProvider(UpstreamProvider):
@@ -718,8 +886,55 @@ class _SimpleUpstreamProvider(UpstreamProvider):
         self.store_dir = store_dir
 
     def provide(self, target_dir):
-        path = (self.already_exists_in_target(target_dir)
+        paths = (self.already_exists_in_target(target_dir)
                 or self.provide_from_store_dir(target_dir))
-        if path is not None:
-            return path
-        raise MissingUpstreamTarball(self._tarball_names()[0])
+        if paths is not None:
+            return [(p, component_from_orig_tarball(p, self.package,
+                self.version)) for p in paths]
+        raise MissingUpstreamTarball(self.package, self.version)
+
+
+class ExtractTarballVersionTests(TestCase):
+
+    def test_unknown_extension(self):
+        self.assertEquals(None,
+            extract_tarball_version("/tmp/foo-1.2.tar.bla", "foo"))
+
+    def test_debian_style(self):
+        self.assertEquals("1.2+testfix",
+            extract_tarball_version("/tmp/foo_1.2+testfix.orig.tar.gz", "foo"))
+
+    def test_traditional_style(self):
+        self.assertEquals("1.2b2",
+            extract_tarball_version("/tmp/foo-1.2b2.zip", "foo"))
+
+
+class NewTarballNameTests(TestCase):
+
+    def test_bz2(self):
+        self.assertEquals("foo_1.0.orig.tar.bz2",
+            new_tarball_name("foo", "1.0", "bla.tar.bz2"))
+
+    def test_gz(self):
+        self.assertEquals("foo_1.0.orig.tar.gz",
+            new_tarball_name("foo", "1.0", "bla.tar.gz"))
+
+    def test_zip(self):
+        self.assertEquals("foo_1.0.orig.tar.gz",
+            new_tarball_name("foo", "1.0", "bla.zip"))
+
+
+class TestGatherOrigTarballs(TestCaseWithTransport):
+
+    def test_no_dir(self):
+        self.assertIs(None, gather_orig_files("mypkg", "1.0", "idontexist"))
+
+    def test_empty(self):
+        self.build_tree(["empty/"])
+        self.assertIs(None, gather_orig_files("mypkg", "1.0", "empty"))
+
+    def test_single(self):
+        self.build_tree(["mypkg_1.0.orig.tar.gz"])
+        self.assertEquals(
+            [os.path.join(self.test_dir, "mypkg_1.0.orig.tar.gz")],
+            gather_orig_files("mypkg", "1.0", "."))
