@@ -19,26 +19,19 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 import os
+import re
 import shutil
 import subprocess
 import tarfile
 import tempfile
 
-from base64 import (
-    standard_b64decode,
-    )
-
-
 try:
     from debian.changelog import Version
 except ImportError:
     # Prior to 0.1.15 the debian module was called debian_bundle
     from debian_bundle.changelog import Version
 
-from bzrlib.errors import (
-    NoSuchRevision,
-    NoSuchTag,
-    )
+from bzrlib import osutils
 from bzrlib.trace import (
     note,
     warning,
@@ -47,18 +40,29 @@ from bzrlib.trace import (
 from bzrlib.plugins.builddeb.errors import (
     MissingUpstreamTarball,
     PackageVersionNotPresent,
-    PerFileTimestampsNotSupported,
-    PristineTarError,
     WatchFileMissing,
     )
-from bzrlib.plugins.builddeb.repack_tarball import repack_tarball
+from bzrlib.plugins.builddeb.repack_tarball import (
+    get_filetype,
+    repack_tarball,
+    )
 from bzrlib.plugins.builddeb.util import (
+    component_from_orig_tarball,
     export,
-    reconstruct_pristine_tar,
     tarball_name,
     )
 
 
+def new_tarball_name(package, version, old_name):
+    """Determine new tarball name based on the package name and the old name.
+    """
+    old_format = get_filetype(old_name)
+    if old_format in ("gz", "bz2", "xz"):
+        return tarball_name(package, version, None, old_format)
+    # Unknown target format, repack to .tar.gz
+    return tarball_name(package, version, None, "gz")
+
+
 class UpstreamSource(object):
     """A source for upstream versions (uscan, get-orig-source, etc)."""
 
@@ -76,6 +80,8 @@ class UpstreamSource(object):
 
         :param package: Package name
         :package version: Version string
+        :raise PackageVersionNotPresent: When the specified version was not
+            found
         """
         raise NotImplementedError(self.version_as_revision)
 
@@ -88,154 +94,25 @@ class UpstreamSource(object):
         """
         raise NotImplementedError(self.has_version)
 
-    def fetch_tarball(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir):
         """Fetch the source tarball for a particular version.
 
         :param package: Name of the package
         :param version: Version string of the version to fetch
         :param target_dir: Directory in which to store the tarball
-        :return: Path of the fetched tarball
+        :return: Paths of the fetched tarballs
         """
-        raise NotImplementedError(self.fetch_tarball)
+        raise NotImplementedError(self.fetch_tarballs)
 
-    def _tarball_path(self, package, version, target_dir, format=None):
-        return os.path.join(target_dir, tarball_name(package, version,
+    def _tarball_path(self, package, version, component, target_dir, format=None):
+        return os.path.join(target_dir, tarball_name(package, version, component,
                     format=format))
 
 
-class PristineTarSource(UpstreamSource):
-    """Source that uses the pristine-tar revisions in the packaging branch."""
-
-    def __init__(self, tree, branch):
-        self.branch = branch
-        self.tree = tree
-
-    def tag_name(self, version, distro=None):
-        """Gets the tag name for the upstream part of version.
-
-        :param version: the Version object to extract the upstream
-            part of the version number from.
-        :return: a String with the name of the tag.
-        """
-        assert isinstance(version, str)
-        if distro is None:
-            return "upstream-" + version
-        return "upstream-%s-%s" % (distro, version)
-
-    def fetch_tarball(self, package, version, target_dir):
-        revid = self.version_as_revision(package, version)
-        try:
-            rev = self.branch.repository.get_revision(revid)
-        except NoSuchRevision:
-            raise PackageVersionNotPresent(package, version, self)
-        note("Using pristine-tar to reconstruct the needed tarball.")
-        if self.has_pristine_tar_delta(rev):
-            format = self.pristine_tar_format(rev)
-        else:
-            format = 'gz'
-        target_filename = self._tarball_path(package, version,
-                                             target_dir, format=format)
-        try:
-            self.reconstruct_pristine_tar(revid, package, version, target_filename)
-        except PristineTarError:
-            raise PackageVersionNotPresent(package, version, self)
-        except PerFileTimestampsNotSupported:
-            raise PackageVersionNotPresent(package, version, self)
-        return target_filename
-
-    def _has_version(self, tag_name, md5=None):
-        if not self.branch.tags.has_tag(tag_name):
-            return False
-        revid = self.branch.tags.lookup_tag(tag_name)
-        self.branch.lock_read()
-        try:
-            graph = self.branch.repository.get_graph()
-            if not graph.is_ancestor(revid, self.branch.last_revision()):
-                return False
-        finally:
-            self.branch.unlock()
-        if md5 is None:
-            return True
-        rev = self.branch.repository.get_revision(revid)
-        try:
-            return rev.properties['deb-md5'] == md5
-        except KeyError:
-            warning("tag %s present in branch, but there is no "
-                "associated 'deb-md5' property" % tag_name)
-            return True
-
-    def version_as_revision(self, package, version):
-        assert isinstance(version, str)
-        for tag_name in self.possible_tag_names(version):
-            if self._has_version(tag_name):
-                return self.branch.tags.lookup_tag(tag_name)
-        tag_name = self.tag_name(version)
-        try:
-            return self.branch.tags.lookup_tag(tag_name)
-        except NoSuchTag:
-            raise PackageVersionNotPresent(package, version, self)
-
-    def has_version(self, package, version, md5=None):
-        assert isinstance(version, str), str(type(version))
-        for tag_name in self.possible_tag_names(version):
-            if self._has_version(tag_name, md5=md5):
-                return True
-        return False
-
-    def possible_tag_names(self, version):
-        assert isinstance(version, str)
-        tags = [self.tag_name(version),
-                self.tag_name(version, distro="debian"),
-                self.tag_name(version, distro="ubuntu"),
-                "upstream/%s" % version]
-        return tags
-
-    def has_pristine_tar_delta(self, rev):
-        return ('deb-pristine-delta' in rev.properties
-                or 'deb-pristine-delta-bz2' in rev.properties)
-
-    def pristine_tar_format(self, rev):
-        if 'deb-pristine-delta' in rev.properties:
-            return 'gz'
-        elif 'deb-pristine-delta-bz2' in rev.properties:
-            return 'bz2'
-        assert self.has_pristine_tar_delta(rev)
-        raise AssertionError("Not handled new delta type in "
-                "pristine_tar_format")
-
-    def pristine_tar_delta(self, rev):
-        if 'deb-pristine-delta' in rev.properties:
-            uuencoded = rev.properties['deb-pristine-delta']
-        elif 'deb-pristine-delta-bz2' in rev.properties:
-            uuencoded = rev.properties['deb-pristine-delta-bz2']
-        else:
-            assert self.has_pristine_tar_delta(rev)
-            raise AssertionError("Not handled new delta type in "
-                    "pristine_tar_delta")
-        return standard_b64decode(uuencoded)
-
-    def reconstruct_pristine_tar(self, revid, package, version,
-            dest_filename):
-        """Reconstruct a pristine-tar tarball from a bzr revision."""
-        tree = self.branch.repository.revision_tree(revid)
-        tmpdir = tempfile.mkdtemp(prefix="builddeb-pristine-")
-        try:
-            dest = os.path.join(tmpdir, "orig")
-            rev = self.branch.repository.get_revision(revid)
-            if self.has_pristine_tar_delta(rev):
-                export(tree, dest, format='dir')
-                delta = self.pristine_tar_delta(rev)
-                reconstruct_pristine_tar(dest, delta, dest_filename)
-            else:
-                export(tree, dest_filename, require_per_file_timestamps=True)
-        finally:
-            shutil.rmtree(tmpdir)
-
-
 class AptSource(UpstreamSource):
     """Upstream source that uses apt-source."""
 
-    def fetch_tarball(self, package, upstream_version, target_dir, 
+    def fetch_tarballs(self, package, upstream_version, target_dir,
             _apt_pkg=None):
         if _apt_pkg is None:
             import apt_pkg
@@ -264,10 +141,17 @@ class AptSource(UpstreamSource):
         lookup = get_fn(sources, 'lookup', 'Lookup')
         while lookup(package):
             version = get_fn(sources, 'version', 'Version')
-            if upstream_version == Version(version).upstream_version:
+            filenames = []
+            for (checksum, size, filename, filekind) in sources.files:
+                if filekind != "tar":
+                    continue
+                filename = os.path.basename(filename)
+                if filename.startswith("%s_%s.orig" % (package, upstream_version)):
+                    filenames.append(filename)
+            if filenames:
                 if self._run_apt_source(package, version, target_dir):
-                    return self._tarball_path(package, version, target_dir)
-                break
+                    return [os.path.join(target_dir, filename)
+                            for filename in filenames]
         note("apt could not find the needed tarball.")
         raise PackageVersionNotPresent(package, upstream_version, self)
 
@@ -287,12 +171,11 @@ class AptSource(UpstreamSource):
 class GetOrigSourceSource(UpstreamSource):
     """Upstream source that uses the get-orig-source rule in debian/rules."""
 
-    def __init__(self, tree, larstiq):
+    def __init__(self, tree, top_level):
         self.tree = tree
-        self.larstiq = larstiq
+        self.top_level = top_level
 
-    def _get_orig_source(self, source_dir, desired_tarball_names,
-                        target_dir):
+    def _get_orig_source(self, source_dir, prefix, target_dir):
         note("Trying to use get-orig-source to retrieve needed tarball.")
         command = ["make", "-f", "debian/rules", "get-orig-source"]
         proc = subprocess.Popen(command, cwd=source_dir)
@@ -300,38 +183,40 @@ class GetOrigSourceSource(UpstreamSource):
         if ret != 0:
             note("Trying to run get-orig-source rule failed")
             return None
-        for desired_tarball_name in desired_tarball_names:
-            fetched_tarball = os.path.join(source_dir, desired_tarball_name)
+        filenames = []
+        for filename in os.listdir(source_dir):
+            if not filename.startswith(prefix):
+                continue
+            fetched_tarball = os.path.join(source_dir, filename)
             if os.path.exists(fetched_tarball):
-                repack_tarball(fetched_tarball, desired_tarball_name,
-                               target_dir=target_dir, force_gz=False)
-                return fetched_tarball
-        note("get-orig-source did not create %s", desired_tarball_name)
+                repack_tarball(fetched_tarball, filename,
+                               target_dir=target_dir)
+                filenames.append(os.path.join(target_dir, filename))
+        if filenames:
+            return filenames
+        note("get-orig-source did not create file with prefix %s", prefix)
         return None
 
-    def fetch_tarball(self, package, version, target_dir):
-        if self.larstiq:
+    def fetch_tarballs(self, package, version, target_dir):
+        if self.top_level:
             rules_name = 'rules'
         else:
             rules_name = 'debian/rules'
         rules_id = self.tree.path2id(rules_name)
         if rules_id is not None:
-            desired_tarball_names = [tarball_name(package, version),
-                    tarball_name(package, version, 'bz2'),
-                    tarball_name(package, version, 'lzma')]
             tmpdir = tempfile.mkdtemp(prefix="builddeb-get-orig-source-")
             try:
                 base_export_dir = os.path.join(tmpdir, "export")
                 export_dir = base_export_dir
-                if self.larstiq:
+                if self.top_level:
                     os.mkdir(export_dir)
                     export_dir = os.path.join(export_dir, "debian")
                 export(self.tree, export_dir, format="dir")
-                tarball_path = self._get_orig_source(base_export_dir,
-                        desired_tarball_names, target_dir)
-                if tarball_path is None:
+                tarball_paths = self._get_orig_source(base_export_dir,
+                        "%s_%s.orig" % (package, version), target_dir)
+                if tarball_paths is None:
                     raise PackageVersionNotPresent(package, version, self)
-                return tarball_path
+                return tarball_paths
             finally:
                 shutil.rmtree(tmpdir)
         note("No debian/rules file to try and use for a get-orig-source rule")
@@ -341,12 +226,12 @@ class GetOrigSourceSource(UpstreamSource):
 class UScanSource(UpstreamSource):
     """Upstream source that uses uscan."""
 
-    def __init__(self, tree, larstiq):
+    def __init__(self, tree, top_level):
         self.tree = tree
-        self.larstiq = larstiq
+        self.top_level = top_level
 
     def _export_watchfile(self):
-        if self.larstiq:
+        if self.top_level:
             watchfile = 'watch'
         else:
             watchfile = 'debian/watch'
@@ -394,7 +279,7 @@ class UScanSource(UpstreamSource):
             os.unlink(tempfilename)
         return self._xml_report_extract_upstream_version(stdout)
 
-    def fetch_tarball(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir):
         note("Using uscan to look for the upstream tarball.")
         try:
             tempfilename = self._export_watchfile()
@@ -406,14 +291,19 @@ class UScanSource(UpstreamSource):
                 "--upstream-version=%s" % version,
                 "--force-download", "--rename", "--package=%s" % package,
                 "--check-dirname-level=0",
-                "--download", "--repack", "--destdir=%s" % target_dir,
+                "--download", "--destdir=%s" % target_dir,
                 "--download-version=%s" % version])
         finally:
             os.unlink(tempfilename)
         if r != 0:
             note("uscan could not find the needed tarball.")
             raise PackageVersionNotPresent(package, version, self)
-        return self._tarball_path(package, version, target_dir)
+        orig_files = gather_orig_files(package, version, target_dir)
+        if orig_files is None:
+            note("the expected files generated by uscan could not be found in"
+                 "%s.", target_dir)
+            raise PackageVersionNotPresent(package, version, self)
+        return orig_files
 
 
 class SelfSplitSource(UpstreamSource):
@@ -436,18 +326,18 @@ class SelfSplitSource(UpstreamSource):
         finally:
             shutil.rmtree(tmpdir)
 
-    def fetch_tarball(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir):
         note("Using the current branch without the 'debian' directory "
                 "to create the tarball")
-        tarball_path = self._tarball_path(package, version, target_dir)
+        tarball_path = self._tarball_path(package, version, None, target_dir)
         self._split(package, version, tarball_path)
-        return tarball_path
+        return [tarball_path]
 
 
 class StackedUpstreamSource(UpstreamSource):
     """An upstream source that checks a list of other upstream sources.
 
-    The first source that can provide a tarball, wins. 
+    The first source that can provide a tarball, wins.
     """
 
     def __init__(self, sources):
@@ -456,15 +346,14 @@ class StackedUpstreamSource(UpstreamSource):
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self._sources)
 
-    def fetch_tarball(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir):
         for source in self._sources:
             try:
-                path = source.fetch_tarball(package, version, target_dir)
+                paths = source.fetch_tarballs(package, version, target_dir)
             except PackageVersionNotPresent:
                 pass
             else:
-                assert isinstance(path, basestring)
-                return path
+                return paths
         raise PackageVersionNotPresent(package, version, self)
 
     def get_latest_version(self, package, version):
@@ -478,6 +367,20 @@ class StackedUpstreamSource(UpstreamSource):
         return None
 
 
+def gather_orig_files(package, version, path):
+    prefix = "%s_%s.orig" % (package, version)
+    ret = []
+    path = os.path.abspath(path)
+    if not os.path.isdir(path):
+        return None
+    for filename in os.listdir(path):
+        if filename.startswith(prefix):
+            ret.append(os.path.join(path, filename))
+    if ret:
+        return ret
+    return None
+
+
 class UpstreamProvider(object):
     """An upstream provider can provide the upstream source for a package.
 
@@ -498,7 +401,7 @@ class UpstreamProvider(object):
         self.source = StackedUpstreamSource(sources)
 
     def provide(self, target_dir):
-        """Provide the upstream tarball any way possible.
+        """Provide the upstream tarball(s) any way possible.
 
         Call this to place the correctly named tarball in to target_dir,
         through means possible.
@@ -525,49 +428,65 @@ class UpstreamProvider(object):
         if in_target is not None:
             note("Upstream tarball already exists in build directory, "
                     "using that")
-            return in_target
+            return [(p, component_from_orig_tarball(p,
+                self.package, self.version)) for p in in_target]
         if self.already_exists_in_store() is None:
             if not os.path.exists(self.store_dir):
                 os.makedirs(self.store_dir)
             try:
-                path = self.source.fetch_tarball(self.package,
+                paths = self.source.fetch_tarballs(self.package,
                     self.version, self.store_dir)
             except PackageVersionNotPresent:
-                raise MissingUpstreamTarball(self._tarball_names()[0])
-            assert isinstance(path, basestring)
+                raise MissingUpstreamTarball(self.package, self.version)
+            assert isinstance(paths, list)
         else:
             note("Using the upstream tarball that is present in %s" %
                  self.store_dir)
-        path = self.provide_from_store_dir(target_dir)
-        assert path is not None
-        return path
+        paths = self.provide_from_store_dir(target_dir)
+        assert paths is not None
+        return [(p, component_from_orig_tarball(p, self.package, self.version))
+                for p in paths]
 
     def already_exists_in_target(self, target_dir):
-        for tarball_name in self._tarball_names():
-            path = os.path.join(target_dir, tarball_name)
-            if os.path.exists(path):
-                return path
-        return None
+        return gather_orig_files(self.package, self.version, target_dir)
 
     def already_exists_in_store(self):
-        for tarball_name in self._tarball_names():
-            path = os.path.join(self.store_dir, tarball_name)
-            if os.path.exists(path):
-                return path
-        return None
+        return gather_orig_files(self.package, self.version, self.store_dir)
 
     def provide_from_store_dir(self, target_dir):
-        path = self.already_exists_in_store()
-        if path is not None:
+        paths = self.already_exists_in_store()
+        if paths is None:
+            return None
+        for path in paths:
             repack_tarball(path, os.path.basename(path),
-                    target_dir=target_dir, force_gz=False)
-            return path
-        return path
+                    target_dir=target_dir)
+        return paths
 
-    def _tarball_names(self):
-        return [tarball_name(self.package, self.version),
-                tarball_name(self.package, self.version, format='bz2'),
-                tarball_name(self.package, self.version, format='lzma')]
+
+def extract_tarball_version(path, packagename):
+    """Extract a version from a tarball path.
+
+    :param path: Path to the tarball (e.g. "/tmp/bzr-builddeb-2.7.3.tar.gz")
+    :param packagename: Name of the package (e.g. "bzr-builddeb")
+    """
+    basename = os.path.basename(path)
+    for extension in [".tar.gz", ".tgz", ".tar.bz2", ".tar.lzma", ".tar.xz",
+            ".zip"]:
+        if basename.endswith(extension):
+            basename = basename[:-len(extension)]
+            break
+    else:
+        # Unknown extension
+        return None
+    # Debian style tarball
+    m = re.match(packagename+"_(.*).orig", basename)
+    if m:
+        return m.group(1)
+    # Traditional, PACKAGE-VERSION.tar.gz
+    m = re.match(packagename+"-(.*)", basename)
+    if m:
+        return m.group(1)
+    return None
 
 
 class TarfileSource(UpstreamSource):
@@ -575,15 +494,94 @@ class TarfileSource(UpstreamSource):
 
     def __init__(self, path, version=None):
         self.path = path
-        # TODO: Parse self.path for version is version is None?
         self.version = version
 
-    def fetch_tarball(self, package, version, target_dir):
+    def fetch_tarballs(self, package, version, target_dir):
         if version != self.version:
             raise PackageVersionNotPresent(package, version, self)
-        dest_name = tarball_name(package, version)
-        repack_tarball(self.path, dest_name, target_dir=target_dir, force_gz=True)
-        return os.path.join(target_dir, dest_name)
+        dest_name = new_tarball_name(package, version, self.path)
+        repack_tarball(self.path, dest_name, target_dir=target_dir)
+        return [os.path.join(target_dir, dest_name)]
 
     def get_latest_version(self, package, version):
-        return self.version
+        if self.version is not None:
+            return self.version
+        return extract_tarball_version(self.path, package)
+
+
+class LaunchpadReleaseFileSource(UpstreamSource):
+    """Source that retrieves release files from Launchpad."""
+
+    @classmethod
+    def from_package(cls, distribution_name, distroseries_name, package):
+        """Create a LaunchpadReleaseFileSource from a distribution package.
+
+        :param distribution_name: Name of the distribution (e.g. "Ubuntu")
+        :param distroseries_name: Name of the distribution series
+            (e.g. "oneiric")
+        :param package: Package name
+        :return: A `LaunchpadReleaseFileSource`
+        """
+        from bzrlib.plugins.builddeb.launchpad import (
+            get_upstream_projectseries_for_package,
+            )
+        project_series = get_upstream_projectseries_for_package(
+            package, distribution_name, distroseries_name)
+        if project_series is None:
+            return None
+        return cls(project_series=project_series)
+
+    def __init__(self, project=None, project_series=None):
+        if project_series is None:
+            self.project_series = project.development_focus
+        else:
+            self.project_series = project_series
+        if project is None:
+            self.project = project_series.project
+        else:
+            self.project = project
+
+    def fetch_tarballs(self, package, version, target_dir):
+        release = self.project.getRelease(version=version)
+        if release is None:
+            raise PackageVersionNotPresent(package, version, self)
+        release_files = []
+        for f in release.files:
+            if f.file_type == "Code Release Tarball":
+                release_files.append(f.file)
+        if len(release_files) == 0:
+            warning("Release %s for package %s found on Launchpad but no "
+                    "associated tarballs.", version, package)
+            raise PackageVersionNotPresent(package, version, self)
+        elif len(release_files) > 1:
+            warning("More than one release file for release %s of package %s"
+                    "found on Launchpad. Using the first.", version, package)
+        hosted_file = release_files[0]
+        tmpdir = tempfile.mkdtemp(prefix="builddeb-get-orig-source-")
+        try:
+            inf = hosted_file.open()
+            try:
+                note("Downloading upstream tarball %s from Launchpad",
+                     inf.filename)
+                filename = inf.filename.encode(osutils._fs_enc)
+                filename = filename.replace("/", "")
+                tmppath = os.path.join(tmpdir, filename)
+                outf = open(tmppath, 'wb')
+                try:
+                    outf.write(inf.read())
+                finally:
+                    outf.close()
+            finally:
+                inf.close()
+            dest_name = new_tarball_name(package, version, filename)
+            repack_tarball(tmppath, dest_name, target_dir=target_dir)
+            return os.path.join(target_dir, dest_name)
+        finally:
+            shutil.rmtree(tmpdir)
+
+    def get_latest_version(self, package, version):
+        versions = []
+        for release in self.project_series.releases:
+            versions.append((release.date_released, release.version))
+        versions.sort()
+        return versions[-1][1].encode("utf-8")
